/*
 * The Good Bookr - PSP Bible Reader
 * Copyright (C) 2007 Phosphorous http://www.pspcoder.net/goodbookr/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <cstring>
#include "bksql.h"
#include "bkuser.h"

// declare static members for linker
vector<BKSql::BKBook> BKSql::books;
string BKSql::searchTerm = "";
string BKSql::errorText = "";

unsigned int BKSql::searchResult;
vector<BKSql::BKBook> BKSql::searchResults;
unsigned int BKSql::searchWithin;
vector<BKSql::BKOption> BKSql::searchWithins;
unsigned int BKSql::searchMatch;
vector<BKSql::BKOption> BKSql::searchMatches;


int BKSql::init(sqlite3 *db){

	if ( books.empty() ){

		string abbrev;
		vector<string> abbrevs;
		sqlite3_stmt *result;
		rc = sqlite3_prepare_v2(db,
			"SELECT book, number, chapters, abbrev FROM books ORDER BY number", -1, &result, 0);

		// loop through results
		while ( (rc = sqlite3_step(result)) == SQLITE_ROW ) {

			// store book details
			books.push_back( BKBook(
				(const char*)sqlite3_column_text(result,0),
				sqlite3_column_int(result,1),
				sqlite3_column_int(result,2)
			) );

			// store book abbreviations for search by verse
			abbrevs.push_back( (const char*)sqlite3_column_text(result,0) );
			abbrev = (const char*)sqlite3_column_text(result,3);
			splitString(abbrev,",",&abbrevs);
			bookAbbrevs.push_back(abbrevs);

		}

		if( rc != SQLITE_DONE ){
			switch(rc){
				case SQLITE_ERROR:
				default:
					errorText = "Unable to load books!";
				break;
			}
			return 0;
		}

		sqlite3_finalize(result);

	}

	// populate search modes for search menu
	searchWithin = 0;
	searchWithins.push_back( BKOption( SEARCH_WITHIN_BOOK, "Current Book" ) );
	searchWithins.push_back( BKOption( SEARCH_WITHIN_OT, "Old Testament" ) );
	searchWithins.push_back( BKOption( SEARCH_WITHIN_PENTATEUCH, "Torah/Pentateuch" ) );
	searchWithins.push_back( BKOption( SEARCH_WITHIN_OT_HISTORY, "OT History" ) );
	searchWithins.push_back( BKOption( SEARCH_WITHIN_OT_POETRY, "OT Poetry" ) );
	searchWithins.push_back( BKOption( SEARCH_WITHIN_OT_MAJOR, "Major Prophets" ) );
	searchWithins.push_back( BKOption( SEARCH_WITHIN_OT_MINOR, "Minor Prophets" ) );
	searchWithins.push_back( BKOption( SEARCH_WITHIN_NT, "New Testament" ) );
	searchWithins.push_back( BKOption( SEARCH_WITHIN_GOSPELS, "Gospels" ) );
	searchWithins.push_back( BKOption( SEARCH_WITHIN_EPISTLES, "Epistles/Letters" ) );
	searchWithins.push_back( BKOption( SEARCH_WITHIN_ALL, "Entire Bible" ) );

	// populate word matching options for search menu
	searchMatch = 0;
	searchMatches.push_back( BKOption( SEARCH_MATCH_PARTIAL, "Partial Word" ) );
	searchMatches.push_back( BKOption( SEARCH_MATCH_FULL, "Full Word" ) );
	searchMatches.push_back( BKOption( SEARCH_MATCH_VERSE, "Verse or Chapter" ) );

	return 1;

}

string BKSql::loadBook(string* text, int bk, int ch, int vs){

	// some bounds checking
	if( bk < 1 ) bk = 1;
	else if( bk > (int)books.size() ) bk = (int)books.size();
	BKUser::options.lastBook = bk;

	int b = bk-1;

	if(ch > books[b].tch) ch = 1;
	else if( ch < 1 ) ch = books[b].tch;
	BKUser::options.lastChapter = ch;

	// build query
	sqlite3_stmt *result;
	rc = sqlite3_prepare(db,
		"SELECT verse,text FROM kjv WHERE book=? AND chapter=?", -1, &result, 0);
	sqlite3_bind_int (result, 1, bk);
	sqlite3_bind_int (result, 2, ch);

	// start text with book and chapter
	char title[100];
	sprintf( title, "%s %i", books[b].name.c_str(), ch );
	text->assign( title );

	// step through query and append verses
	while ( (rc = sqlite3_step(result)) == SQLITE_ROW ) {
		// insert ascii 2 (start of text) as a verse marker
		sprintf( tmpStr, "\n\2%i %s", sqlite3_column_int(result,0), (const char*)sqlite3_column_text(result,1) );
		text->append(tmpStr);
	}

	// work around bookr bug that cuts off last line
	text->append("\n");
	text->append(50,' ');

	// handle errors
	if( rc != SQLITE_DONE ){
		switch(rc){
			case SQLITE_ERROR:
			default:
				errorText = "Unable to load book!";
			break;
		}
		return errorText;
	}

	sqlite3_finalize(result);

	return title;

}

int BKSql::searchBooks(string searchText, int book){

	// build query
	string query = "SELECT book, chapter, verse FROM kjv WHERE";

	// search for specific verse; expecting "gen[esis] 1:1", "1:1", or "1"
	if( searchMatches[searchMatch].key == SEARCH_MATCH_VERSE ){

		// we need two string vectors
		vector<string> queryPieces, versePieces;

		// grab space seperated tokens
		splitString(searchText," ",&queryPieces);

		// figure out the book, otherwise use current book
		book = BKUser::options.lastBook;
		if( !queryPieces.empty() ){

			// assume last piece is chapter/verse so grab, parse, and remove it
			splitString(queryPieces[queryPieces.size()-1],":",&versePieces);
			queryPieces.pop_back();

			// get the full book if it was "1 col", "2 tim", etc.
			if( queryPieces.size() > 1 ){
				queryPieces[0] += (" " + queryPieces[1]);
			}
			// loop through each book's abbreviations to find a match
			int found = 0;
			for( unsigned int i=0; i < bookAbbrevs.size() && !found; i++ ){
				for( unsigned int j=0; j < bookAbbrevs[i].size() && !found; j++ ){
					if( stricmp(bookAbbrevs[i][j].c_str(), queryPieces[0].c_str()) == 0 ){
						book = ++i;
						found = 1;
					}
				}
			}
		}

		// build WHERE clause
		if( !versePieces.empty() ){

			// add book and chapter clause
			sprintf(tmpStr, " book=%i AND chapter=\"%s\"", book, versePieces[0].c_str());
			query.append(tmpStr);

			// add verse clause
			if( versePieces.size() == 2 ){
				sprintf(tmpStr, " AND verse=\"%s\"", versePieces[1].c_str());
				query.append(tmpStr);
			} else {
				query.append(" AND verse=1");
			}

		// fail the query if the text was empty (or all spaces)
		} else {
			query.append(" 0");
		}

		// set book to zero to skip append below
		book = 0;

	// search for complete word
	} else if( searchMatches[searchMatch].key == SEARCH_MATCH_FULL ){

		// escape percent; note: glob is case-sensitive, like is insensitive
		sprintf( tmpStr, " (text LIKE '%s %%' OR text GLOB '* %s[ ,.:;]*')", searchText.c_str(), searchText.c_str() );
		query.append(tmpStr);

	// search for partial word
	} else {

		query.append(" text LIKE '%&&%'");
		query.replace( query.find_first_of("&&"), 2, searchText, 0, searchText.length() );

	}

	if(book > 0){
		if( book < 100 ){
			sprintf( tmpStr, " AND book=%i", book );
			query.append(tmpStr);
		} else {
			// special cases
			switch(book){
				case SEARCH_WITHIN_OT:
					query.append(" AND book<=39"); break;
				case SEARCH_WITHIN_PENTATEUCH:
					query.append(" AND book<=5"); break;
				case SEARCH_WITHIN_OT_HISTORY:
					query.append(" AND book>5 AND book<=17"); break;
				case SEARCH_WITHIN_OT_POETRY:
					query.append(" AND book>17 AND book<=22"); break;
				case SEARCH_WITHIN_OT_MAJOR:
					query.append(" AND book>22 AND book<=27"); break;
				case SEARCH_WITHIN_OT_MINOR:
					query.append(" AND book>27 AND book<=39"); break;
				case SEARCH_WITHIN_NT:
					query.append(" AND book>39 AND book<=66"); break;
				case SEARCH_WITHIN_GOSPELS:
					query.append(" AND book>39 AND book<=43"); break;
				case SEARCH_WITHIN_EPISTLES:
					query.append(" AND book>58 AND book<=65"); break;
			}
		}
	}//errorText = query; return 0; //debug

	// tack on order clause
	query.append(" ORDER BY book ASC");

	// prepare statement
	sqlite3_stmt *result;
	rc = sqlite3_prepare_v2(db, query.c_str(), -1, &result, 0);

	// start query
	rc = sqlite3_step(result);

	// handle errors
	if( rc != SQLITE_ROW ){

		switch(rc){
			case SQLITE_OK:
			case SQLITE_DONE:
				sprintf(tmpStr,"Search for \"%s\" produced no results.",searchText.c_str());
			break;
			case SQLITE_ERROR:
			default:
				sprintf(tmpStr,"Error %i. %s",rc,sqlite3_errmsg(db));
			break;
		}
		errorText = tmpStr;
		return 0;

	// step through query and append verses
	} else {
		searchResults.clear();
		searchTerm = searchText;
		do {
		
			int bk = sqlite3_column_int(result,0);
			int ch = sqlite3_column_int(result,1);
			int vs = sqlite3_column_int(result,2);
			sprintf( tmpStr, "%s %i:%i", books[bk-1].name.c_str(), ch ,vs );
			searchResults.push_back( BKBook(tmpStr, bk, ch, vs) );
		
		} while ( (rc = sqlite3_step(result)) == SQLITE_ROW );
	}

	sqlite3_finalize(result);

	return 1;

}

/* other patterns for SQLite */

//	int rc, nrow, ncol;
//
//	chapter = books[bk].name;
//
//	query = "SELECT verse,text FROM kjv WHERE book=1 AND chapter=1";
//	rc = sqlite3_get_table( db, query.c_str(), &result, &nrow, &ncol, &err );
//	if( rc == SQLITE_OK ){
//		for(int i=ncol; i < ncol*nrow; i+=ncol ){
//			chapter += "\n";
//			chapter += result[i];
//			chapter += " ";
//			chapter += result[i+1];
//		}
//	} else {
//		*text = "Error loading verses!\nSQL error: ";
//		*text += err;
//		sqlite3_free(err);
//		return 0;
//	}
//	sqlite3_free_table(result);
//
//	text->assign(chapter);


//	int rc, nrow, ncol;
//	query = "SELECT book,chapters FROM books";
//	rc = sqlite3_exec(db, query.c_str(), BKSql::initBookDetails, (void*)book, &err);
//	if( rc != SQLITE_OK ){
//		errorText = "Error loading book!\nSQL error: ";
//		errorText += err;
//		sqlite3_free(err);
//		return 0;
//	}
//
// callbacks
//	int BKSql::initBookDetails(void* any, int colCount, char **aColValues, char **aColNames){
//		for(int i=0; i<colCount; i++){
//			if( strcmp("book",aColNames[i]) == 0 ){
//				((BKBOOK)any)[i].name = aColValues[i];
//			} else if( strcmp("chapters",aColNames[i]) == 0 ){
//				((BKBOOK)any)[i].tch = (int)aColValues[i];
//			}
//		}
//		return 0;
//	}
//
//	int BKSql::listFields(void* any, int colCount, char **aColValues, char **aColNames){
//		int i;
//		char tmpStr [1000];
//		sprintf(tmpStr,"\ncolCt = %i", colCount);
//		for(i=0; i<colCount; i++){
//			sprintf(tmpStr,"\n%s = %s", aColNames[i], (aColValues[i] ? aColValues[i] : "NULL") );
//		}
//		return 0;
//	}
